/* Beautify date of postlist item */
.postlist > li > p:nth-child(1) {
  font-size: smaller;
}

/* Big and bold post title for blog index */
.postlist-style-none p:nth-child(1) > .reference {
  font-size: large;
  font-weight: bold;
  font-style: initial;
}

/* Make font size of postlist of index.rst regular */
.postlist-style-disc p:nth-child(1) > .reference {
  font-size: initial;
  font-style: initial;
}

/*********************************
 * Compat with sphinx_book_theme *
 *********************************/

/* Style for combining toctree and postlist of index.rst */
ul.postlist-style-disc {
  padding-left: 1.5em;
}
ul.postlist-style-disc li + li {
   margin-top: 0;
}
ul.postlist-style-disc > li > p:first-child {
   font-size: initial;
}
